﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using SolverInterface;

using System.Threading;

using TSP.Algorithm;

namespace TSPSolverGA
{
    public class TSPSolverGA:ITSPSolver
    {
        public event ExecutionFinishedDelegate ExecutionFinished;
        public event ProgressReachedDelegate ProgressReached;

        private Thread _exectutionWorker;
        private const int GenerationsEstimate = 100;

        public void InitializeAndStart(ICity[] cities)
        {
            // We can use our implementation of City because of it inherites from TSP.Solver.City
            TSP.Algorithm.City[] citiesGA;

            if (cities is TSP.Algorithm.City[]) //Either external aplication use our implementation of City[] which inherits from TSP.Algorithm.City
            {
                citiesGA = (TSP.Algorithm.City[])cities;
            }
            else // Or we have to create new array of cities according the provided array of ICity
            {
                citiesGA = new TSP.Algorithm.City[cities.Length];
                for (int i = 0; i < cities.Length; i++)
                {
                    citiesGA[i] = new City(cities[i]);
                }
            }

            if (_exectutionWorker != null)
            {
                if (_exectutionWorker.IsAlive)
                {
                    _exectutionWorker.Abort();
                }
            }

            _exectutionWorker = new Thread(ComputeTSP);
            _exectutionWorker.Start(citiesGA);
        }

        private void ComputeTSP(object parameters)
        {
            try
            {
                if (!(parameters is TSP.Algorithm.City[]))
                {
                    throw new ArgumentException("Argument must be TSP.Algorithm.City[]");
                }

                TSP.Algorithm.City[] cities = (TSP.Algorithm.City[])parameters;
                GeneticAlgorithm ga = new GeneticAlgorithm();

                ga.Initialize(cities);

                for (int i = 0; i < 1000; i++)
                {
                    ga.Iterate();
                    Console.WriteLine(
                        string.Format("Best cost so far: {0}, in generatin {1}",
                        ga.BestCost, ga.GenerationCount));

                    if (ProgressReached != null)
                    {
                        double progress = (double)i / GenerationsEstimate;
                        ProgressReached(progress, ga.BestCost, ga.BestChromosome.Genes, ga);
                    }
                }

                if (ExecutionFinished != null)
                {
                    ExecutionFinished(ga.BestCost, ga.BestChromosome.Genes, ga);
                }

                Console.WriteLine(
                    string.Format("Overall best cost: {0}, in generatin {1}",
                                    ga.BestCost, ga.GenerationCount));

            }
            catch (ThreadAbortException tex)
            {
                Console.WriteLine(tex.Message);
            }
        }




        public void Dispose()
        {
            ///throw new NotImplementedException();
        }
    }
}
