﻿using System;
using System.Threading;
using System.Windows;
using System.Windows.Media;
using Microsoft.Maps.MapControl.WPF;
using SolverInterface;
using TSPSolverGA;

namespace WpfTSP
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private readonly ICity[] _cities;
        private readonly ITSPSolver _tspSolver;

        public MainWindow()
        {
            InitializeComponent();
            _cities = new[]
                                {
                                    new City(50.083333, 14.416667, "Prague"),
                                    new City(37.566536, 126.977969, "Seoul"),
                                    new City(33.583333, 130.4, "Fukuoka"),
                                    new City(3.1475,101.693333, "Kuala Lumpur"),
                                    new City(5.971389, 116.095278, "Kota Kinabalu"),
                                    new City(48.866667, 2.333333, "Paris"),
                                    new City(51.507222, -0.1275, "London"),
                                    new City(34.05, -118.25, "L.A."),
                                    new City(40.664167, -73.938611, "NYC"),
                                    new City(36.175, -115.136389, "Las Vegas"),
                                    new City(52.500556, 13.398889, "Berlin"),
                                    new City(54.966667, -1.6, "Newcastle upon Tyne"),
                                    new City(47.266667, 11.383333, "Insbruck"),
                                    new City(41.9, 12.5, "Rome"),
                                    new City(37.7793, -122.4192, "San Francisco"),
                                    new City(21.308889, -157.826111, "Honolulu"),
                                };
            _tspSolver = new TSPSolverGA.TSPSolverGA();

            _tspSolver.ExecutionFinished += TSPFinished;
            _tspSolver.ProgressReached += TSPProgressReached;
        }

        private void TSPProgressReached(double progress, double cost, int[] partialsolution, object solver)
        {
            if (myMap.Dispatcher.Thread == Thread.CurrentThread)
            {
                DisplayRoute(partialsolution);
                /*if (solver is GeneticAlgorithm)
                {
                    GeneticAlgorithm ga = (GeneticAlgorithm)solver;

                    infoLabel.Content = string.Format("Best Cost: {1:0.00} km {0}Generations: {2}{0}Last Generation Duration: {3} ms",
                        Environment.NewLine, ga.BestCost, ga.GenerationCount, ga.LastGenerationDuration);
                }
                else
                {
                    infoLabel.Content = string.Format("Best Cost: {0:0.00} km", cost);
                }*/
            }
            else
            {
                myMap.Dispatcher.Invoke(new ProgressReachedDelegate(TSPProgressReached), new[] { progress, cost, partialsolution, solver });
            }
        }


        private void TSPFinished(double cost, int[] solution, object solver)
        {
            if (myMap.Dispatcher.Thread == Thread.CurrentThread)
            {
                //TODO
            }else
            {
                myMap.Dispatcher.Invoke(new ExecutionFinishedDelegate(TSPFinished), new object[] { cost, solution, solver });
            }
        }

        private void DisplayRoute(int[] solution)
        {
            var m = new MapPolyline()
            {
                Stroke = Brushes.Red,
                StrokeThickness = 2,
                Opacity = 0.7,
            };
            m.Locations = new LocationCollection();

            foreach (var gene in solution)
            {
                m.Locations.Add(new Location(_cities[gene].Latitude, _cities[gene].Longitude));
            }

            myMap.Children.Clear();
            myMap.Children.Add(m);
        }

        private void StartTSPButtonClick(object sender, RoutedEventArgs e)
        {
            myMap.Children.Clear();
            _tspSolver.InitializeAndStart(_cities);
        }
    }
}
